function [RepairTime_AllPhase ] = Worker_Allocation(Folder_Path,Repair_Class,n_real,n_Floors,Phases)
% Variable definitions:
% Repair_Class = desired recovery state = e.g.'RC2'
% n_real = number of realizations = 2000
% n_Floors = number of floors in the building = 16
% P1 = lower level from which repairs will be carried out for the phase
% being considered = 5
% P2 = upper level upto which repairs will be carried out for the phase
% being considered = 7

% The repair sequences are defined as per REDi guidelines (2013)
%   RS1: Structural repairs
%   RS2: Interior repairs
%   RS3: Exterior repairs
%   RS4: Mechanical repairs
%   RS5: Electrical repairs
%   RS6: Elevator repairs
%   RS7: Staircase repairs

% This script calculates the repair time after assigning workers for
% each repair sequence based on the following limitations:
%   1) The number of damaged components per floor per repair sequence 
%   (As per Table 2: Improving estimates of Earthquake-Induced Downtime in 
%    Individual Buildings Using The REDi Methodology (2018)).... referred
%    as REDi v2.0 from now on.
%   2) Depending on the repair sequence: Maximum number of workers that can
%   be allocated for each repair sequence across all floors
%   (Table 1 of REDi v 2.0)
%   3) Based on Story Floor Area: 1 worker per 500sft if building is
%   unoccupied during repairs and 1 worker per 1000sft if building is
%   occupied
%   4) Based on Building Floor Area: REDi guidelines (2013)

% Following assumptions are considered when allocating workers:
%   1) Structural and non-structural repair can happen simultaneously 
%   2) A contractor repairs 2-3 floors simultaneously and then moves on to the next set of floors
%   3) Interior and equipment repairs start only after structural repairs on that floor are complete

% Data imports              ( Read data carefully and make neccessary modifications to extents, file names, building floor area )

%1) Number of damaged components per repair sequence per floor for desired recovery state
% Size: m x n
% m rows = n_Floors x no. of repair sequences
% n columns = number of realizations
% Filename depends on the recovery state considered
file = 'RS_Number_DC_';
file_type = '.xlsx';
filename = append(Folder_Path,file,Repair_Class,file_type);
RS_Number_DC = xlsread(filename,1,'A1:BXX119');

%2) Repair time matrix per repair sequence and per floor for desired recovery state
% Size: m x n
% m rows = n_Floors x no. of repair sequences
% n columns = number of realizations
file2 = 'RS_Repair_Times_';
filename2 = append(Folder_Path,file2,Repair_Class,file_type);
Repair_Times_All = xlsread(filename2,1,'A1:BXX119');

%3) Limitations on worker allocation for each repair sequence 
filename3 = append(Folder_Path,'Worker_Limitations.xlsx');
Labor_Limitations = xlsread(filename3,1,'B2:D8');

%4) Repair Class matrix 
filename4 = append(Folder_Path,'Repair_Class_All.xlsx');
Repair_Class_All = xlsread(filename4,1,'A1:BXX715');

%5) Floor area of the building in sft
Floor_Area = 100*100;

n_Phase = size(Phases,1);
WorkersAllocation_AllPhase = zeros(n_Floors*7,n_real);
RepairTime_AllPhase = zeros(n_Floors*7,n_real);

for phase=1:n_Phase
    
P1 = Phases(phase,1);       % Bottom floor in phase
P2 = Phases(phase,2);       % Top floor in phase

% Number of floors that are considered in this repair phase
n_Floor_Group = P2-P1+1;

WL_DC_RS1 = zeros(n_Floor_Group,n_real);
WL_DC_RS2 = zeros(n_Floor_Group,n_real);
WL_DC_RS3 = zeros(n_Floor_Group,n_real);
WL_DC_RS4 = zeros(n_Floor_Group,n_real);
WL_DC_RS5 = zeros(n_Floor_Group,n_real);
WL_DC_RS6 = zeros(n_Floor_Group,n_real);
WL_DC_RS7 = zeros(n_Floor_Group,n_real);

RT_P_RS1 = zeros(n_Floor_Group,n_real);
RT_P_RS2 = zeros(n_Floor_Group,n_real);
RT_P_RS3 = zeros(n_Floor_Group,n_real);
RT_P_RS4 = zeros(n_Floor_Group,n_real);
RT_P_RS5 = zeros(n_Floor_Group,n_real);
RT_P_RS6 = zeros(n_Floor_Group,n_real);
RT_P_RS7 = zeros(n_Floor_Group,n_real);

for i=1:n_real
    p=1;
    for j=P1:P2
        k=j-1;
        % The worker limit based on the number of damaged components:  Paul et al(2018) Table 2
        WL_DC_RS1(p,i) = ceil(RS_Number_DC(7*k+1,i)*Labor_Limitations(1,1));
        WL_DC_RS2(p,i) = ceil(RS_Number_DC(7*k+2,i)*Labor_Limitations(2,1));
        WL_DC_RS3(p,i) = ceil(RS_Number_DC(7*k+3,i)*Labor_Limitations(3,1));
        WL_DC_RS4(p,i) = ceil(RS_Number_DC(7*k+4,i)*Labor_Limitations(4,1));
        WL_DC_RS5(p,i) = ceil(RS_Number_DC(7*k+5,i)*Labor_Limitations(5,1));
        WL_DC_RS6(p,i) = ceil(RS_Number_DC(6,i)*Labor_Limitations(6,1));        %Because elevators damaged component are only located on the ground floor  
        WL_DC_RS7(p,i) = ceil(RS_Number_DC(7*k+7,i)*Labor_Limitations(7,1));
           
        % The repair time (from FEMA P-58 curves) for the selected group of
        % floors based on repair sequence:
        RT_P_RS1(p,i) = Repair_Times_All(7*k+1,i);
        RT_P_RS2(p,i) = Repair_Times_All(7*k+2,i);
        RT_P_RS3(p,i) = Repair_Times_All(7*k+3,i);
        RT_P_RS4(p,i) = Repair_Times_All(7*k+4,i);
        RT_P_RS5(p,i) = Repair_Times_All(7*k+5,i);
        RT_P_RS6(p,i) = Repair_Times_All(7*k+6,i);
        RT_P_RS7(p,i) = Repair_Times_All(7*k+7,i);
        
        p=p+1;
        
    end
end 


%The total number of workers permitted based on the type of repair
WL_RS1 = Labor_Limitations(1,2);
WL_RS2 = Labor_Limitations(2,2);
WL_RS3 = Labor_Limitations(3,2);
WL_RS4 = Labor_Limitations(4,2);
WL_RS5 = Labor_Limitations(5,2);
WL_RS6 = Labor_Limitations(6,2);
WL_RS7 = Labor_Limitations(7,2);

% The total number of workers that can be allocated throughout the building
% across all repair sequences based on the building floor area
WL_bldg = ceil((2.5e-4*Floor_Area*n_Floors) + 10);

% The sequence in which the repairs are given priority for worker
% allocations is: 
%   RS1: Structural repairs
%   RS7: Staircase repairs
%   RS6: Elevator repairs
%   RS3: Exterior repairs
%   RS2: Interior repairs
%   RS4: Mechanical repairs
%   RS5: Electrical repairs


% Calculate the number of workers that can be assigned 
Workers_RS1 = zeros(n_Floor_Group,n_real);
Workers_RS7 = zeros(n_Floor_Group,n_real);
Workers_RS6 = zeros(n_Floor_Group,n_real);
Workers_RS3 = zeros(n_Floor_Group,n_real);
Workers_RS2 = zeros(n_Floor_Group,n_real);
Workers_RS4 = zeros(n_Floor_Group,n_real);
Workers_RS5 = zeros(n_Floor_Group,n_real);

for i=1:n_real
    
    % Calculate the worker limit based on the floor area
    % 1 worker per 500sft if building is unoccupied during repairs and 1 worker per 1000sft if building is occupied
    
    % If RC4 is triggered, the building is unoccupied and the worker limit
    % is 1 worker per 500 sft. If RC4 is not triggered i.e. the building
    % can be used as a shelter, the worker limit is reduced to 1 worker per
    % 1000 sft because residents will also be using the building
    if max(Repair_Class_All(:,i)) >= 4
       Labor_Limitations_FA = 0.002;
    else
        Labor_Limitations_FA = 0.001;
    end
    Worker_limit_FA= Floor_Area * Labor_Limitations_FA;
    
    % Check on floor area
    for j=1:n_Floor_Group
        Workers_RS1(j,i) = min(WL_DC_RS1(j,i),Worker_limit_FA);
        Workers_RS7(j,i) = min(WL_DC_RS7(j,i),Worker_limit_FA);
        Workers_RS6(j,i) = min(WL_DC_RS6(j,i),Worker_limit_FA);
        Workers_RS3(j,i) = min(WL_DC_RS3(j,i),Worker_limit_FA);
        
        temp1 = sum([WL_DC_RS2(j,i),WL_DC_RS4(j,i),WL_DC_RS5(j,i)]);      % Since these sequences happen together, check if total workers in these sequences exceed floor limit
        if temp1 > Worker_limit_FA
            Workers_RS2(j,i) = floor(WL_DC_RS2(j,i)/temp1*Worker_limit_FA);
            Workers_RS4(j,i) = floor(WL_DC_RS4(j,i)/temp1*Worker_limit_FA);
            Workers_RS5(j,i) = floor(WL_DC_RS5(j,i)/temp1*Worker_limit_FA);
        else 
            Workers_RS2(j,i) = WL_DC_RS2(j,i);
            Workers_RS4(j,i) = WL_DC_RS4(j,i);
            Workers_RS5(j,i) = WL_DC_RS5(j,i);
        end
    end
    
    % Check on max workers per repair sequence (Paul et al(2018) Table 1)
    if sum(Workers_RS1(:,i)) > WL_RS1
        for j=1:n_Floor_Group
            Workers_RS1(j,i) = floor(Workers_RS1(j,i)/sum(Workers_RS1(:,i))*WL_RS1);
        end
    end
    if sum(Workers_RS2(:,i)) > WL_RS2
        for j=1:n_Floor_Group
            Workers_RS2(j,i) = floor(Workers_RS2(j,i)/sum(Workers_RS2(:,i))*WL_RS2);
        end
    end
    if sum(Workers_RS3(:,i)) > WL_RS3
        for j=1:n_Floor_Group
            Workers_RS3(j,i) = floor(Workers_RS3(j,i)/sum(Workers_RS3(:,i))*WL_RS3);
        end
    end
    if sum(Workers_RS4(:,i)) > WL_RS4
        for j=1:n_Floor_Group
            Workers_RS4(j,i) = floor(Workers_RS4(j,i)/sum(Workers_RS4(:,i))*WL_RS4);
        end
    end
    if sum(Workers_RS5(:,i)) > WL_RS5
        for j=1:n_Floor_Group
            Workers_RS5(j,i) = floor(Workers_RS5(j,i)/sum(Workers_RS5(:,i))*WL_RS5);
        end
    end
    if Workers_RS6(1,i) > WL_RS6       % Because elevator workers will be only be allocated to one floor at a time, we check limit only against first floor
        for j=1:n_Floor_Group
            Workers_RS6(j,i) = floor(Workers_RS6(j,i)/sum(Workers_RS6(:,i))*WL_RS6);
        end
    end
    if sum(Workers_RS7(:,i)) > WL_RS7
        for j=1:n_Floor_Group
            Workers_RS7(j,i) = floor(Workers_RS7(j,i)/sum(Workers_RS7(:,i))*WL_RS7);
        end
    end
    
    
    % Check on max workers in building (REDi (2013)) % We do not account
    % for RS6 in this limitation because RS6 workers are only assigned on
    % one floor in each phase
    temp2 = sum(Workers_RS1(:,i)+Workers_RS2(:,i)+Workers_RS3(:,i)+Workers_RS4(:,i)+Workers_RS5(:,i)+Workers_RS7(:,i));
    
    if temp2 > WL_bldg
        for j=1:n_Floor_Group
            Workers_RS1(j,i) = floor(Workers_RS1(j,i)/temp2*WL_bldg);
            Workers_RS2(j,i) = floor(Workers_RS2(j,i)/temp2*WL_bldg);
            Workers_RS3(j,i) = floor(Workers_RS3(j,i)/temp2*WL_bldg);
            Workers_RS4(j,i) = floor(Workers_RS4(j,i)/temp2*WL_bldg);
            Workers_RS5(j,i) = floor(Workers_RS5(j,i)/temp2*WL_bldg);
            Workers_RS6(j,i) = floor(Workers_RS6(j,i)/temp2*WL_bldg);
            Workers_RS7(j,i) = floor(Workers_RS7(j,i)/temp2*WL_bldg);
        end
    end 
end

%% Repair time after accounting for worker limitations
Summary_RT = zeros(7*n_Floor_Group,n_real);
for i=1:n_real
    for j=1:n_Floor_Group
        p=j-1;
    %RS1
    if Workers_RS1(j,i) ~=0
        Summary_RT(7*p+1,i) = RT_P_RS1(j,i) ./ Workers_RS1(j,i);
    else
        Summary_RT(7*p+1,i) = RT_P_RS1(j,i);    
    end
    %RS2
    if Workers_RS2(j,i) ~=0
        Summary_RT(7*p+2,i) = RT_P_RS2(j,i) ./ Workers_RS2(j,i);
    else
        Summary_RT(7*p+2,i) = RT_P_RS2(j,i);    
    end
    %RS3
    if Workers_RS3(j,i) ~=0
        Summary_RT(7*p+3,i) = RT_P_RS3(j,i) ./ Workers_RS3(j,i);
    else
        Summary_RT(7*p+3,i) = RT_P_RS3(j,i);    
    end
    %RS4
    if Workers_RS4(j,i) ~=0
        Summary_RT(7*p+4,i) = RT_P_RS4(j,i) ./ Workers_RS4(j,i);
    else
        Summary_RT(7*p+4,i) = RT_P_RS4(j,i);    
    end
    %RS5
    if Workers_RS5(j,i) ~=0
        Summary_RT(7*p+5,i) = RT_P_RS5(j,i) ./ Workers_RS5(j,i);
    else
        Summary_RT(7*p+5,i) = RT_P_RS5(j,i);    
    end
    %RS6
    % Adjustment for workers allocated for elevator repair
    % We do not consider elevator repairs in each floor will happen in parallel, but we consider that they workers allocated will work in series. Because the number of workers allocated is equal across all floors, we make the following sumple adjustment 
    if j==1
        Summary_RT(7*p+6,i) = sum(RT_P_RS6(:,i)) ./ Workers_RS6(j,i);
    else
        Summary_RT(7*p+6,i) = 0;    
    end
    
    
    %RS7
    if Workers_RS7(j,i) ~=0
        Summary_RT(7*p+7,i) = RT_P_RS7(j,i) ./ Workers_RS7(j,i);
    else
        Summary_RT(7*p+7,i) = RT_P_RS7(j,i);    
    end
    end
end

Summary_Workers = zeros(7*n_Floor_Group,n_real);
for i=1:n_real
    for j=1:n_Floor_Group
        p=j-1;
        Summary_Workers(7*p+1,i)=Workers_RS1(j,i); 
        Summary_Workers(7*p+2,i)=Workers_RS2(j,i); 
        Summary_Workers(7*p+3,i)=Workers_RS3(j,i); 
        Summary_Workers(7*p+4,i)=Workers_RS4(j,i); 
        Summary_Workers(7*p+5,i)=Workers_RS5(j,i); 
        Summary_Workers(7*p+6,i)=Workers_RS6(j,i); 
        Summary_Workers(7*p+7,i)=Workers_RS7(j,i); 
    end
end

WorkersAllocation_AllPhase((P1-1)*7+1 : P2*7,:) = Summary_Workers;
RepairTime_AllPhase((P1-1)*7+1 : P2*7,:) = Summary_RT;

end

% We need the worker allocations for functional recovery in order to plot
% the deaggregation of mean downtime to functional recovery
if Repair_Class == 'RC2'
    xlswrite(append(Folder_Path,'Worker_Allocation_RC2.xlsx'),WorkersAllocation_AllPhase)
    xlswrite(append(Folder_Path,'Reduced_RepairTime_RC2.xlsx'),RepairTime_AllPhase)
end


end
